require 'test/unit'
require 'Money'
require 'Dollar'
require 'Franc'
require 'Bank'
require 'Sum'

class MoneyTest < Test::Unit::TestCase

  def test_multiplication
     five = Money.new.dollar 5
     assert (Money.new.dollar 10).equals(five.times 2)
     assert (Money.new.dollar 15).equals(five.times 3)
     
   end
  
  def test_equality
     assert (Money.new.dollar 5).equals(Money.new.dollar 5)
     assert !(Money.new.dollar 5).equals(Money.new.dollar 6)
     assert !(Money.new.franc 5).equals(Money.new.dollar 5)
  end
  
  def test_franc_multiplication
    five = Money.new.franc 5
    assert (Money.new.franc 10).equals(five.times 2)
    assert (Money.new.franc 15).equals(five.times 3)
  end
  
  def test_currency
    assert_equal("USD",(Money.new.dollar 1).currency)
    assert_equal("CHF",(Money.new.franc 1).currency)
  end
  
  def test_different_class_equality
    assert (Money.new 10,"CHF").equals(Franc.new 10,"CHF")
  end
  def test_simple_addition
    five = Money.new.dollar 5
    sum = five.plus five
    bank = Bank.new
    reduced = bank.reduce sum,"USD"
    assert (Money.new.dollar 10).equals(reduced)
  end
  def test_plus_returns_sum
    five = Money.new.dollar 5
    sum = five.plus five
    assert five.equals(sum.augend)
    assert five.equals(sum.addend)
  end
  def test_reduce_sum
    sum = Sum.new((Money.new.dollar 3),(Money.new.dollar 4))
    bank = Bank.new
    result = bank.reduce(sum,"USD")
    assert (Money.new.dollar 7).equals(result)
  end
  def test_reduce_money
    bank = Bank.new
    result = bank.reduce((Money.new.dollar 1),"USD")
    assert (Money.new.dollar 1).equals(result)
  end
  def test_reduce_money_different_currency
    bank = Bank.new
    bank.add_rate "CHF","USD",2
    result = bank.reduce((Money.new.franc 2),"USD")
    assert (Money.new.dollar 1).equals(result)
  end
  def test_mixed_addition
    five_bucks = Money.new.dollar 5
    ten_francs = Money.new.franc 10
    bank = Bank.new
    bank.add_rate "CHF","USD",2
    result = bank.reduce((five_bucks.plus(ten_francs)),"USD")
    assert (Money.new.dollar 10).equals(result)
  end
  def test_sum_plus_money
    five_bucks = Money.new.dollar 5
    ten_francs = Money.new.franc 10
    bank = Bank.new
    bank.add_rate "CHF","USD",2
    sum = Sum.new(five_bucks,ten_francs).plus(five_bucks)
    result = bank.reduce(sum,"USD")
    assert (Money.new.dollar 15).equals(result)
  end
  def test_sum_times
    five_bucks = Money.new.dollar 5
    ten_francs = Money.new.franc 10
    bank = Bank.new
    bank.add_rate "CHF","USD",2
    sum = Sum.new(five_bucks,ten_francs).times(2)
    result = bank.reduce(sum,"USD")
    assert (Money.new.dollar 20).equals(result)
  end
 end